import sys
from PyQt5 import QtCore, QtWidgets
from vtk.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor
from vedo import Mesh, dataurl, Plotter
from vedo.pyplot import np, plot

try:
    _encoding = QtWidgets.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtWidgets.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtWidgets.QApplication.translate(context, text, disambig)


class Ui_MainWindow(object):

    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)

        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")

        self.tab1 = QtWidgets.QWidget()
        self.tab1.setObjectName("tab1")
        self.gridLayout1 = QtWidgets.QGridLayout(self.tab1)

        self.vtkLayout1 = QtWidgets.QVBoxLayout()
        self.vtkLayout1.setObjectName("vtkLayout1")
        self.gridLayout1.addLayout(self.vtkLayout1, 0, 0, 1, 1)

        self.tab2 = QtWidgets.QWidget()

        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.addTab(self.tab1, "tab1")
        self.tabWidget.addTab(self.tab2, "tab2")

        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.addWidget(self.tabWidget)

        self.gridLayout2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout2.addLayout(self.verticalLayout, 0, 0, 1, 1)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 31))

        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        MainWindow.setStatusBar(self.statusbar)

        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow", None))
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.tab1), _translate("MainWindow", "Tab 1", None)
        )
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.tab2), _translate("MainWindow", "Tab 2", None)
        )

        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)


class MainWindow(QtWidgets.QMainWindow, Ui_MainWindow):

    def __init__(self, parent=None):
        QtWidgets.QMainWindow.__init__(self, parent)
        self.setupUi(self)

        self.vtkWidget1 = QVTKRenderWindowInteractor(self)
        self.vtkLayout1.addWidget(self.vtkWidget1)

        self.plt1 = Plotter(qt_widget=self.vtkWidget1, axes=1)
        self.id2 = self.plt1.add_callback("key press", self.onKeypress)
        self.plt1 += Mesh(dataurl+'shark.ply').c('cyan')
        self.plt1.show()

        self.vtkWidget2 = QVTKRenderWindowInteractor(self)
        self.verticalLayout.addWidget(self.vtkWidget2)
        self.plt2 = Plotter(qt_widget=self.vtkWidget2)

        #####################################################
        # add a plot using a formatted Figure
        x = np.random.randn(100) + 10
        y = np.random.randn(100) * 20 + 20
        fig = plot(
            x, y,
            lw=0,         # do not join points with lines
            xtitle="variable x",
            ytitle="variable y",
            marker="*",   # marker style
            mc="dr",      # marker color
            aspect=16/9,  # aspect ratio
        )
        self.plt2 += fig

        ##################################################### show
        self.plt2.show(zoom=1.8, mode='image')

    def onClose(self):
        self.vtkWidget1.close()

    def onKeypress(self, evt):
        print("You have pressed key:", evt.keypress)
        if evt.keypress=='q':
            sys.exit(0)


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    window = MainWindow()
    app.aboutToQuit.connect(window.onClose)
    window.show()
    sys.exit(app.exec_())



"""Geodesic regression for data in SO(3)
with the Morphometrics library"""
try:
    from morphomatics.manifold import SO3
    from morphomatics.stats import RiemannianRegression
except ModuleNotFoundError:
    print("Install with:")
    print("pip install git+https://github.com/morphomatics/morphomatics.git#egg=morphomatics")
import numpy as np
import vedo

# z-axis is axis of rotation
I = np.eye(3)
R = np.array(
    [[np.cos(np.pi / 6), -np.sin(np.pi / 6), 0],
    [np.sin( np.pi / 6),  np.cos(np.pi / 6), 0],
    [0, 0, 1]],
)

# 6 points in SO(3). The extra dimension is not needed here but comes
# into play when the data consists of tuples of matrices.
M = SO3()
Y = np.zeros((6,) + tuple(M.point_shape))  # -> (6,1,3,3)
eval_perturbed = lambda t, vec: M.connec.exp(M.connec.geopoint(I,R,t), vec)
Y[0, 0] = eval_perturbed(-2 / 3, np.array([[0, 0, 0.1], [0, 0, 0.0], [-0.1, 0.0, 0]]))
Y[1, 0] = eval_perturbed(-1 / 3, np.array([[0, 0, 0.0], [0, 0, 0.2], [ 0.0,-0.2, 0]]))
Y[2, 0] = I
Y[3, 0] = eval_perturbed( 1 / 3, np.array([[0, 0, 0.0], [0, 0, 0.2], [ 0.0,-0.2, 0]]))
Y[4, 0] = eval_perturbed( 2 / 3, np.array([[0, 0, 0.1], [0, 0, 0.0], [-0.1, 0.0, 0]]))
Y[5, 0] = R

# corresponding time points
t = np.array([0, 1/5, 2/5, 3/5, 4/5, 1])

# geodesic has degree 1
degrees = np.array([1])
# cubic regression
# degrees = np.array([3])

# solve
regression = RiemannianRegression(M, Y, t, degrees)

# geodesic least-squares estimator
gam = regression.trend

# evaluate geodesic at 100 equidistant points
X = gam.eval()

# rotate [1,0,0] by rotations in X, i.e. take first column of X
x = X[...,0].squeeze()
time = np.linspace(0,1, x.shape[0])
pts = [y[..., 0][0] for y in Y]

# visualize
pts = vedo.Points(pts, r=15)
line = vedo.Line(x).lw(10).cmap("jet", time)
line.add_scalarbar("time")
sphere = vedo.Sphere(c='white').flat()
vedo.show(sphere, line, pts, __doc__, axes=1, viewup='z').close()




"""Find the intersection points of two coplanar lines"""
import numpy as np
from vedo import *

p1, p2 = (-1,-1,0), (10,2,0)

x = np.linspace(0,10, 50)
y = np.sin(x)*4
pts = np.c_[x,y]

# create 2 lines and assign some arbitrary rotations
line1 = Spline(pts).lw(5).c('black').rotate_y(10).rotate_x(15)
line2 = Line(p1,p2).lw(5).c('green').rotate_y(10).rotate_x(15)

# make a small extrusion of line1 and intersect it with line2:
ds = line1.diagonal_size()*0.02  # 1% tolerance
pint = line1.extrude(ds).shift(0,0,-ds/2).intersect_with_line(line2)
ps = Points(pint, r=15).c('red')

# lets fill the convex area between the first 2 hits:
id0 = line1.closest_point(pint[0], return_point_id=True)
id1 = line1.closest_point(pint[1], return_point_id=True)
msh = Line(line1.points()[id0:id1]).triangulate().lw(0).shift(0,0,-0.01)

show(line1, line2, ps, msh, __doc__+f"\narea = {msh.area()} cm:^2", axes=1).close()



from vedo import Points, Axes, show
import numpy as np

pts = np.random.randn(2000,3)*[3,2,4]-[1,2,3]
vpts1 = Points(pts).alpha(0.2).c('blue2')
vpts2 = vpts1.clone().shift(5,6,7).c('green2')

axs = Axes(
    [vpts1, vpts2],  # build axes for this set of objects
    xtitle="X-axis in :mum",
    ytitle="Variable Y in :mum",
    ztitle="_inverted Z in :mum",
    htitle='My :Gamma^2_ijk  plot',
    htitle_font='Kanopus',
    htitle_justify='bottom-right',
    htitle_color='red2',
    htitle_size=0.035,
    htitle_offset=(0,0.075,0),
    htitle_rotation=45,
    zhighlight_zero=True,
    xyframe_line=2, yzframe_line=1, zxframe_line=1,
    xyframe_color='red3',
    xyshift=1.05, # move xy 5% above the top of z-range
    yzgrid=True,
    zxgrid=True,
    zxshift=1.0,
    xtitle_justify='bottom-right',
    xtitle_offset=-1.175,
    xlabel_offset=-1.75,
    ylabel_rotation=90,
    z_inverted=True,
    tip_size=0.25,
)

show(vpts1, vpts2, axs, "Customizing Axes", viewup='z').close()



from vedo import *

# Set the number of colors to generate
n = 256

# Initialize some variables
i, grids, vnames1, vnames2 = 0, [], [], []

# Loop over all available colormap names
for name in colors.cmaps_names:
    # Skip reversed maps
    if '_r' in name:
        continue
    
    # Generate a list of n RGB color values for the colormap
    cols = color_map(range(n), name)

    gr = Grid(s=[50,1], res=[n,1])
    gr.cellcolors = cols*255
    gr.linewidth(0).wireframe(False).y(-i*1.2)
    grids.append([gr, gr.box().c('grey')])

    # Add a text label with the colormap name to the left of the strip
    tx1 = Text3D(':rightarrow '+name, justify='left-center', s=0.75, font=2)
    tx1.pos(gr.xbounds(1), gr.y()).c('w')
    tx2 = tx1.clone(deep=False).c('k')
    vnames1.append(tx1)
    vnames2.append(tx2)
    i += 1

printc("Try picking a color by pressing Shift-i", invert=True)

# Create a plotter with two renderers
plt = Plotter(N=2, size=(1300,1000))

# Show the grids with the white text labels
plt.at(0).show(grids, vnames1, bg='blackboard')
plt.at(1).show(grids, vnames2, bg='white', mode='image', zoom='tight')

# Enable interactivity and display the plot, then close it
plt.interactive().close()



"""Extract the mesh region that
has the largest connected surface"""
from vedo import dataurl, Volume, printc, Plotter

mesh1 = Volume(dataurl+"embryo.tif").isosurface(80).c("yellow")
printc("area1 =", mesh1.area(), c="yellow")

mesh2 = mesh1.extract_largest_region().color("lb")
printc("area2 =", mesh2.area(), c="lb")

plt = Plotter(shape=(2,1), axes=7)
plt.at(0).show(mesh1, __doc__)
plt.at(1).show(mesh2)
plt.interactive().close()



"""Visualize a TetMesh with
default ray casting."""
from vedo import *

# settings.use_depth_peeling = False

tetm = TetMesh(dataurl+'limb_ugrid.vtk')
tetm.color('jet').alpha_unit(100) # make the tets more transparent
tetm.add_scalarbar3d()

# Build a Mesh object made of all the boundary triangles
wmesh = tetm.tomesh(fill=False).wireframe()

# Make a copy of tetm and shrink the tets
shrunk = tetm.clone().shrink(0.5)

# Build a Mesh object and cut it
cmesh = shrunk.tomesh(fill=True)

show([(tetm, __doc__),
      (wmesh, "..wireframe surface"),
      (cmesh, "..shrunk tetrahedra"),
     ], N=3, axes=1,
).close()




#Credits:
#M. Attene. A lightweight approach to repairing digitized polygon meshes.
#The Visual Computer, 2010. (c) Springer. DOI: 10.1007/s00371-010-0416-3
#http://pymeshfix.pyvista.org
#TetGen, a Delaunay-Based Quality Tetrahedral Mesh Generator
#https://github.com/pyvista/tetgen
#
# pip install pymeshfix
# pip install tetgen
#
import pymeshfix
import tetgen
import vedo

vedo.settings.use_depth_peeling = True

amesh = vedo.Mesh(vedo.dataurl+'290.vtk')

# repairing also closes the mesh in a nice way
meshfix = pymeshfix.MeshFix(amesh.points(), amesh.faces())
meshfix.repair()
repaired = vedo.Mesh(meshfix.mesh).linewidth(1).alpha(0.5)

# tetralize the closed surface
tet = tetgen.TetGen(repaired.points(), repaired.faces())
tet.tetrahedralize(order=1, mindihedral=20, minratio=1.5)
tmesh = vedo.TetMesh(tet.grid)

# save it to disk
#tmesh.write("my_tetmesh.vtk")

plt = vedo.Plotter(N=3, axes=1)
plt.at(0).show("Original mesh", amesh)
plt.at(1).show("Repaired mesh", repaired)
plt.at(2).show("Tetrahedral mesh\n(click & press shift-X)", tmesh.tomesh().shrink())
plt.interactive().close()




"""Compute 3D world coordinates from 2D screen pixel coordinates
(hover mouse to place the points)"""
from vedo import *

settings.default_font = "Ubuntu"
settings.use_depth_peeling = True


def func(evt):                   # this is the callback function
    i = evt.at                   # the renderer nr. which is being hit
    pt2d = evt.picked2d          # 2D screen coordinate
    # passing a list of meshes will force the points to be placed on any of them
    pt3d = plt.at(i).compute_world_coordinate(pt2d, objs=[objs[i]])
    if mag(pt3d) < 0.01:
        return
    newpt = Point(pt3d).color(i)
    txt.text(f'2D coords: {pt2d}\n3D coords: {pt3d}\nNpt = {len(plt.actors)}')
    txt.color(i)                  # update text and color on the fly
    plt.at(i).add(newpt).render() # add new point and render i


# create an empty text (to be updated in the callback)
txt = Text2D("", s=1.4, font='Brachium', c='white', bg='green8')

# create two polygonal meshes
mesh1 = TessellatedBox()
mesh2 = ParametricShape('ConicSpiral')
mesh2.c('indigo1').lc('grey9').lw(0.1)
objs = [mesh1, mesh2]

plt = Plotter(N=2, bg='blackboard', axes=1, sharecam=False)
plt.add_callback('mouse move', func)

plt.at(0).show(mesh1, __doc__, viewup='z')
plt.at(1).show(mesh2, txt, zoom=1.4)
plt.interactive().close()




"""Create a scatter plot to overlay
three different distributions"""
from vedo import *
from numpy.random import randn

### first cloud in blue, place it at z=0:
x = randn(2000) * 3
y = randn(2000) * 2
pts1 = Points([x, y], c="blue", alpha=0.5).z(0.0)
bra1 = Brace([-7, -8], [7, -8], comment="whole population", s=0.4, c="b")

### second cloud in red
x = randn(1200) + 4
y = randn(1200) + 2
pts2 = Points([x, y], c="red", alpha=0.5).z(0.1)
bra2 = Brace(
    [8, 2, 0.3],
    [6, 5, 0.3],
    comment="red zone",
    angle=180,
    justify="bottom-center",
    c="r",
)

### third cloud with a black marker
x = randn(20) + 4
y = randn(20) - 4
mark = Marker("*", s=0.25)
pts3 = Glyph([x, y], mark, c="k").z(0.2)
bra3 = Brace([8, -6], [8, -2], comment="my stars").z(0.3)

# some text message
msg = Text3D("preliminary\nresults!", font="Quikhand", s=1.5)
msg.c("black").rotate_z(20).pos(-10, 3, 0.2)

show(
    pts1, pts2, pts3, msg, bra1, bra2, bra3, __doc__,
    axes=1, zoom=1.2, mode="image",
).close()



"""Stream tubes originating from a probing grid of points.
Data is from CFD analysis of airflow in an office with
ventilation and a burning cigarette"""
from vedo import *
from off_furniture import furniture

fpath = download(dataurl + 'office.binary.vtk')
sgrid = loadStructuredGrid(fpath)

# Create a grid of points and use those as integration seeds
seeds = Grid(res=[2,3], c="gray").rotate_y(90).pos(2,2,1)

# Now we will generate multiple streamlines in the data
slines = StreamLines(
    sgrid,
    seeds,
    initial_step_size=0.01,
    max_propagation=15,
    tubes=dict(radius=0.005, mode=2, ratio=1),
)
slines.cmap("Reds").add_scalarbar3d(c='white')
slines.scalarbar.x(5) # reposition scalarbar at x=5

show(slines, seeds, furniture(), __doc__, axes=1, bg='bb').close()



"""Superimpose histograms and curves"""
import numpy as np
from vedo.pyplot import histogram, plot
from vedo import settings

settings.default_font = "Bongas"
settings.remember_last_figure_format = True

mu, sigma, n, bins = 100.0, 15.0, 600, 50
samples = np.random.normal(loc=mu, scale=sigma, size=n)
x = np.linspace(min(samples), max(samples), num=50)
y = 1/(sigma*np.sqrt(2*np.pi)) * np.exp( -(x-mu)**2 /(2*sigma**2))
dy = 1/np.sqrt(n)

fig = histogram(
    samples,
    title=__doc__,
    bins=bins,
    density=True,
    c='cyan3',
    aspect=9/7,
    label="gaussian",
)

fig += plot(x, y, "-", lc='orange5', label="some fit")
fig += plot(x, y*(1+dy), "--", c='orange5', lw=2)
fig += plot(x, y*(1-dy), "--", c='orange5', lw=2)

fig.add_legend()
fig.show(size=(800,700), zoom="tight").close()



"""Customizing axes style
(40+ control parameters!)
Title font: """
from vedo import Box, Lines, Points, Spline, show, settings

settings.default_font = 'Theemim'

# an invisible box:
world = Box(pos=(2.7,0,0), size=(12,10,8), alpha=0)

# a dummy spline with its shadow on the xy plane
pts = Points([(-2,-3.2,-1.5), (3,-1.2,-2), (7,3,4)], r=12)
spl = Spline(pts, res=50).add_shadow(plane='z', point=-4) # make spline and add its shadow at z=-4
lns = Lines(spl, spl.shadows[0])                          # join spline points with its own shadow

# make a dictionary of axes options
axes_opts = dict(
    xtitle='My variable :Omega^:lowerxi_lm  in units of :mum^3', # latex-style syntax
    ytitle='This is my highly\ncustomized y-axis',
    ztitle='z in units of Å', # many unicode chars are supported (type: vedo -r fonts)
    y_values_and_labels=[(-3.2,'Mark^a_-3.2'), (-1.2,'Carmen^b_-1.2'), (3,'John^c_3')],
    text_scale=1.3,       # make all text 30% bigger
    number_of_divisions=5, # approximate number of divisions on longest axis
    axes_linewidth= 2,
    grid_linewidth= 1,
    zxgrid2=True,        # show zx plane on opposite side of the bounding box
    yzgrid2=True,        # show yz plane on opposite side of the bounding box
    xyplane_color='green7',
    xygrid_color='dg',    # darkgreen line color
    xyalpha=0.2,         # grid opacity
    xtitle_position=0.5,  # title fractional positions along axis
    xtitle_justify="top-center", # align title wrt to its axis
    ytitle_size=0.02,
    ytitle_box=True,
    ytitle_offset=0.05,
    ylabel_offset=0.4,
    yhighlight_zero=True, # draw a line highlighting zero position if in range
    yhighlight_zero_color='red',
    zline_color='blue',
    ztitle_color='blue',
    ztitle_backface_color='v', # violet color of axis title backface
    label_font="Quikhand",
    ylabel_size=0.025,    # size of the numeric labels along Y axis
    ylabel_color='dg',    # color of the numeric labels along Y axis
)

show(world, pts, spl, lns, __doc__+settings.default_font, axes=axes_opts).close()




"""Align a set of curves in space
with Procrustes method"""
from vedo import *

# Load splines from a file (returns a list of vedo.Lines)
splines = load(dataurl+'splines.npy')

# Perform Procrustes alignment on the splines, allowing for non-rigid transformations
procus = procrustes_alignment(splines, rigid=False)

# Unpack the aligned splines from the Assembly object into a Python list
alignedsplines = procus.unpack()

# Obtain the mean spline and create a Line object with thicker width and blue color
mean = procus.info['mean']
lmean = Line(mean, lw=4, c='b').z(0.001) # z-shift it to make it visible

# Color the aligned splines based on their distance from the mean spline
for l in alignedsplines:
    darr = mag(l.points()-mean)  # distance array
    l.cmap('hot_r', darr, vmin=0, vmax=0.007)

# Add the mean spline and script description to the list of aligned splines
alignedsplines += [lmean, __doc__]

# Show the original and aligned splines in two side-by-side views 
# with independent cameras
show([splines, alignedsplines], N=2, sharecam=False, axes=1).close()




import numpy as np
import vedo
from optics_base import Lens, Ray, Mirror, Detector, Screen  # see file ./optics_base.py


###################################################################### thin lenses
s = vedo.Sphere(r=2, res=50)                      # construct a thin lens:
shape = s.boolean("intersect", s.clone().z(3.5)).z(1.4)
lens  = Lens(shape, ref_index=1.52).color("orange9")
screen= Screen(3,3).z(5)

elements = [lens, screen]
source = vedo.Disc(r1=0, r2=0.7, res=4).points()  # numpy 3d points
lines = [Ray(pt).trace(elements).asLine() for pt in source]  # list of vedo.Line

vedo.show("Test of  1/f = (n-1) \dot (1/R1-1/R2) \approx 1/2",
          elements, lines, lens.boundaries().lw(2),
          azimuth=-90, zoom=1.2, size=(1100,700), axes=dict(zxgrid=True)).close()


####################################################################### dispersion
s = vedo.Cone(res=4).scale([1,1,0.4]).rotate_y(-90).rotate_x(45).pos(-0.5,0,1.5)
prism = Lens(s, ref_index="glass").lw(1)
screen= Screen(2,2).z(6)
lines = []
for wl in np.arange(450,750, 10)*1e-09:
    ray = Ray(direction=(-0.5,0,1), wave_length=wl)
    line = ray.trace([prism,screen]).asLine()
    lines.append(line)
vedo.show("Test of chromatic dispersion", prism, screen, lines,
          zoom=1.5, size=(1100,700), axes=1).close()


################################################################ spherical mirrors
s1 = vedo.Sphere(r=7, res=50).cut_with_plane([0,0,6],'z').cut_with_cylinder(invert=True)
s2 = vedo.Sphere(r=5, res=50).cut_with_plane([0,0,-2],'-z').cut_with_cylinder().z(10)
m1 = Mirror(s1)
m2 = Mirror(s2)
screen = Screen(5,5).z(9)
elements = [m2, m1, m2,  m1, screen] ## NOTE ordering!
source= vedo.Disc(r1=1, r2=3, res=[20,60]).cut_with_plane().cut_with_plane(normal='y').z(1)
lines = [Ray(pt).trace(elements).asLine(2) for pt in source.points()]
vedo.show("Reflection from spherical mirrors", elements, lines, axes=1).close()


################################################################# parabolic mirror
s = vedo.Paraboloid(res=200).cut_with_plane([0,0,-0.4], 'z').scale([1,1,0.1]).z(1)
elements = [Mirror(s), Screen(0.2,0.2).z(0.35)]
source= vedo.Disc(r1=.1, r2=.3, res=[10,30]).cut_with_plane().cut_with_plane(normal='y')
lines = [Ray(pt).trace(elements).asLine() for pt in source.points()]
vedo.show("Reflection from a parabolic mirror", elements, lines, axes=2, azimuth=-90).close()


################################################################# mesh mirror
# Create the mirror from a vedo.Mesh object
shape = vedo.Mesh(vedo.dataurl+"bunny.obj").fill_holes().subdivide().smooth()
shape.scale(7).pos(0.1,-0.6,0).rotate_x(90)
mirror = Mirror(shape).color("silver")

# Create a detector surface as a quad-sphere surrounding the shape
sd = vedo.Sphere(quads=1, res=12).cut_with_plane([0,-0.8,0], normal='y')
detector = Detector(sd).color("white").alpha(1).lw(1)

source = vedo.Grid(res=[30,30]).rotate_x(90).y(-1)
lines=[]
for pt in source.points():
    ray = Ray(pt, direction=(0,1,0)).trace([mirror, detector])
    line = ray.asLine(min_hits=2, max_hits=4)
    lines.append(line)

detector.count().cmap("Reds", on='cells', vmax=10).add_scalarbar("Counts")

vedo.show(mirror, detector, lines, "A Mesh mirror and a spherical detector",
          elevation=-90, axes=1, bg='bb', bg2='blue9').close()


# ################################################################# interference
s1 = vedo.Sphere(res=100).rotate_y(90).cut_with_plane([0,0,0.9], normal='z').y(-.5)
s2 = vedo.Sphere(res=100).rotate_y(90).cut_with_plane([0,0,0.9], normal='z').y(+.5)
src = vedo.merge(s1,s2).clean().compute_normals()
dirs = src.pointdata["Normals"]
screen= Screen(3,3).z(4)

grid = vedo.Grid(res=[40,40], s=[4,4]).rotate_z(90)
detector = Detector(grid).z(3.5)

elements = [detector]
rays, lines, pols = [], [], []
for i,pt in enumerate(src.points()):
    ray = Ray(pt, direction=dirs[i], wave_length=1).trace(elements) # radio waves
    line = ray.asLine()
    if not i%20:
        lines.append(line)
    pols.append(ray.polarizations[-1])

detector.integrate(pols).cmap("brg", on='cells').add_scalarbar("Prob.")

vedo.show("Interference on a detector surface", s1,s2, lines, elements,
          zoom=1.5, size=(1100,700), elevation=180, azimuth=90, axes=1).close()





from vedo import Volume, dataurl
from vedo.applications import RayCastPlotter

embryo = Volume(dataurl+"embryo.slc").mode(1).c('jet')  # change properties

plt = RayCastPlotter(embryo, bg='black', bg2='blackboard', axes=7)  # Plotter instance

plt.show(viewup="z").close()



"""Animation of a block attached to a spring"""
from vedo import *
from vedo.applications import AnimationPlayer

L = 0.1    # spring x position at rest
x0 = 0.85  # initial x-coordinate of the block
k = 20     # spring constant
m = 20     # block mass
b = 5      # viscosity friction (proportional to velocity)
dt = 0.15  # time step

v  = vector(0, 0, 0.3)  # initial conditions
x  = vector(x0, 0, 0)
xr = vector(L, 0, 0)
x0 = vector(-0.8, 0, 0)

# Pre-compute the trajectory of the block and store it in a list.
history_x = []
for i in range(200):
    F = -k * (x - xr) - b * v         # force and friction
    a = F / m                         # acceleration
    v = v + a * dt                    # velocity
    x = x + v * dt + 1/2 * a * dt**2  # position
    history_x.append(x)

# Create the objects to be shown in the animation
floor = Box(pos=(0, -0.1, 0), size=(2.0, 0.02, 0.5), c='yellow2')
wall  = Box(pos=(-0.82, 0.15, 0), size=(0.04, 0.50, 0.3), c='yellow2')
block = Cube(pos=x, side=0.2, c="tomato")
spring= Spring(x0, x, r1=0.05, thickness=0.005)
text  = Text2D(font="Calco", c='white', bg='k', alpha=1, pos='top-right')

# Create the animation player and it's callback function
def update_scene(i: int):
    # update block and spring position at frame i
    block.pos(history_x[i])
    spring.stretch(x0, history_x[i])
    text.text(f"Frame number {i}\nx = {history_x[i][0]:.4f}")
    plt.render()

plt = AnimationPlayer(update_scene, irange=[0,200], loop=True)
plt += [floor, wall, block, spring, text, __doc__]
plt.set_frame(0)
plt.show()
plt.close()



"""Include background images in the rendering scene
(e.g. generated by matplotlib)"""
import matplotlib.pyplot as plt
from vedo import dataurl, show, Mesh, Picture2D

msh = Mesh(dataurl+"limb_ugrid.vtk").shrink(0.8)

# Create a histogram with matplotlib
fig = plt.figure()
plt.hist(msh.celldata["chem_0"], log=True)
plt.title(r'$\mathrm{Matplotlib\ Histogram\ of\ log(chem_0)}$')

pic1 = Picture2D(fig, scale=0.5, pos="bottom-right").ontop()
pic2 = Picture2D(dataurl+"images/embryo.jpg", pos='top-right')

show(msh, pic1, pic2, __doc__, bg='lightgrey', axes=1)



from vedo import *
from vedo.pyplot import histogram, plot

cmap = 'nipy_spectral'
alpha = np.array([0, 0, 0.05, 0.2, 0.8, 1])

vol = Volume(dataurl+"embryo.slc")
vol.cmap(cmap).alpha(alpha).add_scalarbar3d(c='white')
xvals = np.linspace(*vol.scalar_range(), len(alpha))

fig = histogram(vol, logscale=True, c=cmap, ac='white')
fig+= plot(xvals, alpha*max(fig.frequencies), '--ow', like=fig).z(1)

show([
      (vol, Axes(vol, c='w'), f"Original Volume\ncolor map: {cmap}"),
      (fig, "Voxel scalar histogram\nand opacity transfer function")
     ],
     N=2, sharecam=False, bg=(82,87,110), zoom=1.1,
).close()



"""Find the vertices that are connected
to a specific vertex in a mesh"""
from vedo import *

# create a wireframe sphere and color it yellow
s = Sphere(c="y", res=12).wireframe()

# select one point on the sphere using its index
index = 12
pt = s.points()[index]

# find all the vertices that are connected to the selected point
ids = s.connected_vertices(index)
vtxs = s.points()[ids]

# create a red point at the selected point's location
apt = Point(pt, c="r", r=15)

# create blue points at the locations of the vertices 
# connected to the selected point
cpts = Points(vtxs, c="blue", r=20)

# show the sphere, the selected point, and the connected vertices
show(s, apt, cpts, __doc__, bg='bb').close()
